"""
    The main entry point to the Report library.
    Copyright (C) 2009 Red Hat, Inc

    Author: Gavin Romig-Koch <gavin@redhat.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""

import sys
import os
import os.path
import glob
import imputil
import tempfile


import gettext
_ = lambda x: gettext.ldgettext("report", x)

#
#  A Signature, for the purposes of this library, is a mapping of names
#  to values (in Python terms a Dictionary).  For maximum portablility,
#  names must be ASCII alpha-numeric characters.  Values should be 
#  types that conform to the SignatureValue api below.
# 

class SignatureValue:
    # roughly an arbitrary value that can be collected, stored, and 
    # shipped over the wire.
    # 
    # a SignatureValue can be any type that conforms to this protocol
    #
    # asString() - return a string representation of the data
    # asFile() - return a file representation of the data
    # asFileName() - return the name of a file (on the local file
    #                system) that contains the data, can be a temporary
    #                file
    # 
    # isBinary - boolean attribute, true if the data can not be dumped as 
    #                               is into XML or email.
    pass

class StringSignatureValue:
    def __init__(self, data):
        self.data = data
        self.isBinary = False
        self.filename = None

    def asString(self):
        return self.data

    def asFile(self):
        return file(self.asFileName())

    def asFileName(self):
        if not self.filename:
            if self.isBinary:
                mymode = 'w+b'
            else:
                mymode = 'w+'
            file = tempfile.NamedTemporaryFile(mode=mymode,delete=False,prefix="report-")
            file.write(self.data)
            self.filename = file.name
            file.close()
        return self.filename

    def __del__(self):
        if self.filename:
            os.remove(self.filename)
            self.filename = None

class NamedFileSignatureValue:
    def __init__(self, filename, isBinary):
        self.filename = filename
        self.isBinary = isBinary

    def asString(self):
        return file(self.filename).read()

    def asFile(self):
        return file(self.filename)

    def asFileName(self):
        return self.filename

class FileSignatureValue:
    def __init__(self, afile, isBinary):
        self.afile = afile
        self.isBinary = isBinary

    def asString(self):
        return self.afile.read()

    def asFile(self):
        return self.afile

    def asFileName(self):
        return self.afile.name

class ChoiceValue:
    def __init__(self,title,explanation,returnValue):
        self.title = title
        self.explanation = explanation
        self.returnValue = returnValue

class IO:
    # IO is a callback mechinism for communicating with the user
    # IO can be any type that conforms to the following protocol
    # def infoMessage(self,title,msg):
    #   display message, title and msg are strings
    #   return nothing
    # def failMessage(self,title,msg):
    #   display an error message, title and msg are strings
    #   return nothing
    # def successMessage(self,title,msg,actualURL,displayURL)
    #   display a sucess message, all args are strings,
    #     displayURL and actualURL should both refer to the same 
    #     internet resource, displayURL is for display to the user, 
    #     actualURL is for if you want to link to the resource.
    #     If actualURL is empty but displayURL is not, displayURL
    #     is shown but not as a link.
    #     if displayURL is empty but actualURL is not, displayURL
    #       defaults to actualURL
    #     displayURL and actualURL can be the same string.
    #   return nothing
    # def queryLogin(self,account):
    #   Ask the user for the username and password for logging into
    #     account (a string).
    #   return a dictionary which contains at least two members with
    #     the keys "username" and "password".  The values of these members
    #     should be strings.
    # def updateLogin(self,account,loginResult):
    #   Update the login information for account.
    #   If you call queryLogin, and the login is then successfull,
    #     call this function with the result of the queryLogin to
    #     tell the account manager that the login was successfull
    #   returns nothing
    # def queryField(self,fieldName)
    #     asks for a string value, returns string value
    # def queryChoice(self,msg,choices):
    #    msg is a message about the choices
    #    choices is a sequence of ChoiceValues
    #      Each ChoiceValue (title,explanation,returnValue)
    #    returns the returnValue of the choice the user made
    pass

def createAlertSignature(component, hashmarkername, hashvalue, summary, alertSignature):
    return { "component" : StringSignatureValue(component),
             "hashmarkername" : StringSignatureValue(hashmarkername),
             "localhash" : StringSignatureValue(hashvalue),
             "summary" : StringSignatureValue(summary),
             "description" : StringSignatureValue(alertSignature) }


def createPythonUnhandledExceptionSignature(component, hashmarkername, hashvalue, summary, description, exnFileName):
    return { "component" : StringSignatureValue(component),
             "hashmarkername" : StringSignatureValue(hashmarkername),
             "localhash" : StringSignatureValue(hashvalue),
             "summary" : StringSignatureValue(summary),
             "description" : StringSignatureValue(description),
             "pythonUnhandledException" : NamedFileSignatureValue(exnFileName,False) }


def report(signature, io):

    found = imputil.imp.find_module("alternatives",sys.modules[__name__].__path__)
    alternatives = imputil.imp.load_module("alternatives", found[0], found[1], found[2])


    moduleNames = set(map(lambda x: os.path.splitext(os.path.basename(x))[0],
                          glob.glob(alternatives.__path__[0] + "/*")))
    choices = ()

    for moduleName in moduleNames:
        if moduleName == "__init__":
            continue

        found = imputil.imp.find_module(moduleName,alternatives.__path__)
        if found:
            module = imputil.imp.load_module(moduleName, found[0], found[1], found[2])
            choices += (ChoiceValue(module.label,module.description,module.report),)
        
    choice = io.queryChoice("Where do you want to send this report:",
                            choices)

    if choice is None:
        return False

    else:
        return choice(signature,io)

